package pendo.io.pendo_sdk;

import android.app.Activity;
import android.app.Application;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.util.HashMap;
import android.util.Log;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import sdk.pendo.io.Pendo;

/** PendoFlutterPlugin */
public class PendoFlutterPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {
  final static String pendoChannelName = "pendo_sdk_channel_name";
  private static final String API_CALL_INIT_SDK              = "initSDK";
  private static final String API_CALL_CLEAR_VISITOR         = "clearVisitor";
  private static final String API_CALL_SWITCH_VISITOR        = "switchVisitor";
  private static final String API_CALL_SET_VISITOR_DATA = "setVisitorData";
  private static final String API_CALL_SET_ACCOUNT_DATA      = "setAccountData";
  private static final String API_CALL_INIT_WITHOUT_VISITOR  = "initSDKWithoutVisitor";
  private static final String API_CALL_TRACK                 = "track";
  private static final String API_CALL_END_SESSION           = "endSession";
  private static final String API_CALL_RESUME_GUIDES         = "resumeGuides";
  private static final String API_CALL_DISMISS_VISIBLE_GUIDE = "dismissVisibleGuides";
  private static final String API_CALL_SET_ACCOUNT_ID        = "setAccountId";
  private static final String API_CALL_PAUSE_GUIDES          = "pauseGuides";
  private static final String API_CALL_GET_VISITOR_ID        = "getVisitorId";
  private static final String API_CALL_GET_DEVICE_ID         = "getDeviceId";
  private static final String API_CALL_GET_ACCOUNT_ID        = "getAccountId";
  private static final String API_CALL_SETUP                 = "setup";
  private static final String API_CALL_START_SESSION         = "startSession";
  private static final String API_SET_DEBUG_MODE             = "setDebugMode";

  private static final String APP_KEY              = "ApplicationKey";
  private static final String APP_PARAM            = "ApplicationParameters";
  private static final String KEY_VISITOR_ID       = "VisitorIDKey";
  private static final String KEY_ACCOUNT_ID       = "AccountIDKey";
  private static final String KEY_VISITOR_DATA     = "VisitorDataKey";
  private static final String KEY_ACCOUNT_DATA     = "AccountDataKey";
  private static final String KEY_EVENT_NAME       = "EventNameKey";
  private static final String KEY_DISMISS_GUIDES   = "DismissGuidesKey";
  private static final String KEY_EVENT_PROPERTIES = "EventPropertiesKey";
  private static final String KEY_IS_DEBUG_ENABLED = "isDebugEnabled";

  private static final String VAL_VISITOR_ID      = "visitorId";
  private static final String VAL_ACCOUNT_ID      = "accountId";
  private static final String VAL_VISITOR_DATA    = "visitorData";
  private static final String VAL_ACCOUNT_DATA    = "accountData";
  private static final String VAL_ENV_NAME        = "environmentName";

  

  private Activity mActivity;

  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel channel;

  public PendoFlutterPlugin() {
    super();
  }
  private PendoFlutterPlugin(Activity activity) {
    super();
    this.mActivity = activity;
  }

  /**  FlutterPlugin **/
  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), pendoChannelName);
    channel.setMethodCallHandler(this);
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), pendoChannelName);
    channel.setMethodCallHandler(new PendoFlutterPlugin(registrar.activity()));
  }


  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    channel.setMethodCallHandler(null);
  }

  /**  ActivityAware **/
  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
    this.mActivity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {
    this.mActivity = null;
  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {
    this.mActivity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivity() {
    this.mActivity = null;
  }

  /**  MethodCallHandler **/
  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    switch (call.method) {
      case API_CALL_INIT_SDK:
        initSDK(call, result);
        break;

      case API_CALL_CLEAR_VISITOR:
        clearVisitor(result);
        break;

      case API_CALL_SWITCH_VISITOR:
        switchVisitor(call, result);
        break;

      case API_CALL_START_SESSION:
        startSession(call, result);
        break;

      case API_CALL_SET_VISITOR_DATA:
        setVisitorData(call, result);
        break;

      case API_CALL_SET_ACCOUNT_DATA:
        setAccountData(call, result);
        break;

      case API_CALL_INIT_WITHOUT_VISITOR:
        initSdkWithoutVisitor(call, result);
        break;

      case API_CALL_SETUP:
        setup(call, result);
        break;

      case API_CALL_TRACK:
        track(call, result);
        break;

      case API_CALL_END_SESSION:
        endSession(result);
        break;

      case API_CALL_RESUME_GUIDES:
        resumeGuide(result);
        break;

      case API_CALL_DISMISS_VISIBLE_GUIDE:
        dismissVisibleGuides(result);
        break;

      case API_CALL_SET_ACCOUNT_ID:
        setAccountId(call, result);
        break;
        
      case API_CALL_PAUSE_GUIDES:
        pauseGuides(call, result);
        break;

      case API_CALL_GET_VISITOR_ID:
        getVisitorID(result);
        break;

      case API_CALL_GET_ACCOUNT_ID:
        getAccountId(result);
        break;

      case API_CALL_GET_DEVICE_ID:
        getDeviceId(result);
        break;

      case API_SET_DEBUG_MODE:
        setDebugMode(call, result);
        break;

      default:
        methodNotImplemented(result);
        break;
    }
  }

  private void methodNotImplemented(Result result) {
    result.notImplemented();
  }

  private void setDebugMode(MethodCall call, Result result) {
    if (call.arguments instanceof HashMap) {
      HashMap arguments = (HashMap) call.arguments;
      Object isDebugEnabled = arguments.get(KEY_IS_DEBUG_ENABLED);
      if(isDebugEnabled != null) {
        Pendo.setDebugMode((boolean) isDebugEnabled);
      } else {
        Log.w("Pendo", "setDebugMode - did not receive a boolean value");
      }
    }
    result.success(null);
  }

  private void getDeviceId(Result result) {
    result.success(Pendo.getDeviceId());
  }

  private void getAccountId(Result result) {
    result.success(Pendo.getAccountId());
  }

  private void getVisitorID(Result result) {
    result.success(Pendo.getVisitorId());
  }

  private void pauseGuides(MethodCall call, Result result) {
    if (call.arguments instanceof HashMap) {
      HashMap arguments = (HashMap) call.arguments;
      Object dismissGuides = arguments.get(KEY_DISMISS_GUIDES);
      if(dismissGuides != null) {
        Pendo.pauseGuides((boolean) dismissGuides);
      } else {
        Log.w("Pendo", "pauseGuides - did not receive a boolean value");
      }
    }
    result.success(null);
  }

  private void setAccountId(MethodCall call, Result result) {
    if (call.arguments instanceof HashMap) {
      HashMap arguments = (HashMap) call.arguments;
      Object accountId = arguments.get(KEY_ACCOUNT_ID);
      Pendo.setAccountId((String) accountId);
    }
    result.success(null);
  }

  private void dismissVisibleGuides(Result result) {
    Pendo.dismissVisibleGuides();
    result.success(null);
  }

  private void resumeGuide(Result result) {
    Pendo.resumeGuides();
    result.success(null);
  }

  private void endSession(Result result) {
    Pendo.endSession();
    result.success(null);
  }

  private void track(MethodCall call, Result result) {
    if (call.arguments instanceof HashMap) {
      HashMap arguments = (HashMap) call.arguments;
      Object eventName = arguments.get(KEY_EVENT_NAME);
      Object properties = arguments.get(KEY_EVENT_PROPERTIES);
      if(eventName != null) {
        if (isNullOrWhiteSpace((String) eventName)) {
          Log.e("Pendo", "Track - Event name is required");
        } else {
          Pendo.track((String) eventName, (HashMap) properties);
        }
      }
    }
    result.success(null);
  }

  private void setup(MethodCall call, Result result) {
    if (call.arguments instanceof HashMap) {
      HashMap arguments = (HashMap) call.arguments;
      Object appKey = arguments.get(APP_KEY);
      Object appParams = arguments.get(APP_PARAM);
      if(appKey!= null) {
        Pendo.PendoOptions pendoOptions = createPendoOptions((HashMap) appParams);
        Pendo.setup(this.mActivity, (String) appKey, pendoOptions, null);
      } else {
        Log.e("Pendo", "Setup - appKey is missing");
      }
    }
    result.success(null);
  }

  private void initSdkWithoutVisitor(MethodCall call, Result result) {
    if (call.arguments instanceof HashMap) {
      HashMap arguments = (HashMap) call.arguments;
      Object appKey = arguments.get(APP_KEY);
      Object appParams = arguments.get(APP_PARAM);
      if(appKey!= null) {
        Pendo.PendoOptions pendoOptions = createPendoOptions((HashMap) appParams);
        Pendo.setup(this.mActivity, (String) appKey, pendoOptions, null);
      } else {
        Log.e("Pendo", "initSdkWithoutVisitor - appKey is missing");
      }
    }
    result.success(null);
  }

  private void setAccountData(MethodCall call, Result result) {
    if (call.arguments instanceof HashMap) {
      HashMap arguments  = (HashMap) call.arguments;
      Object accountData = arguments.get(KEY_ACCOUNT_DATA);
      Pendo.setAccountData((HashMap) accountData);
    }
    result.success(null);
  }

  private void setVisitorData(MethodCall call, Result result) {
    if (call.arguments instanceof HashMap) {
      HashMap arguments = (HashMap) call.arguments;
      Object visitorData = arguments.get(KEY_VISITOR_DATA);
      Pendo.setVisitorData((HashMap) visitorData);
    }
    result.success(null);
  }

  private void startSession(MethodCall call, Result result) {
    if (call.arguments instanceof HashMap) {
      HashMap arguments = (HashMap) call.arguments;
      Object visitorId = arguments.get(KEY_VISITOR_ID);
      Object accountId = arguments.get(KEY_ACCOUNT_ID);
      Object visitorData = arguments.get(KEY_VISITOR_DATA);
      Object accountData = arguments.get(KEY_ACCOUNT_DATA);
      Pendo.startSession((String) visitorId, (String) accountId, (HashMap) visitorData, (HashMap) accountData);
    }
    result.success(null);
  }

  private void switchVisitor(MethodCall call, Result result) {
    if (call.arguments instanceof HashMap) {
      HashMap arguments = (HashMap) call.arguments;
      Object visitorId = arguments.get(KEY_VISITOR_ID);
      Object accountId = arguments.get(KEY_ACCOUNT_ID);
      Object visitorData = arguments.get(KEY_VISITOR_DATA);
      Object accountData = arguments.get(KEY_ACCOUNT_DATA);
      Pendo.switchVisitor((String) visitorId, (String) accountId, (HashMap) visitorData, (HashMap) accountData);
    }
    result.success(null);
  }

  private void clearVisitor(Result result) {
    Pendo.clearVisitor();
    result.success(null);
  }

  private void initSDK(MethodCall call, Result result) {
    if (call.arguments instanceof HashMap) {
      HashMap arguments  = (HashMap) call.arguments;
      Object appKey = arguments.get(APP_KEY);
      Object initParams = arguments.get(APP_PARAM);
      if(appKey != null) {
        Pendo.PendoInitParams initParamsObject = createInitParams((HashMap) initParams);
        Pendo.initSDK(this.mActivity, (String) appKey, initParamsObject);   // call initSDK with initParams as a third parameter (this can be `null`).
      } else {
        Log.e("Pendo", "initSDK - appKey is missing");
      }
    }
    result.success(null);
  }

  private Pendo.PendoInitParams createInitParams(@Nullable HashMap params) {
    Pendo.PendoInitParams initParams = new Pendo.PendoInitParams();
    initParams.setPendoOptions(new Pendo.PendoOptions.Builder().setIgnoreFirstOnCreate(true).build());
    if (params != null) {
      if (params.containsKey(VAL_VISITOR_ID)) {
        initParams.setVisitorId((String) params.get(VAL_VISITOR_ID));
      }
      if (params.containsKey(VAL_ACCOUNT_ID)) {
        initParams.setAccountId((String) params.get(VAL_ACCOUNT_ID));
      }
      if (params.containsKey(VAL_VISITOR_DATA)) {
        initParams.setVisitorData((HashMap<String, Object>)params.get(VAL_VISITOR_DATA));
      }
      if (params.containsKey(VAL_ACCOUNT_DATA)) {
        initParams.setAccountData((HashMap<String, Object>)params.get(VAL_ACCOUNT_DATA));
      }
      if (params.containsKey(VAL_ENV_NAME)) {
        String environmentName = (String)params.get(VAL_ENV_NAME);
        if (environmentName != "") {
          initParams.setPendoOptions(new Pendo.PendoOptions.Builder().setEnvironmentDebugOnly(environmentName).build());
        }
      }
    }
    return initParams;
  }

  private Pendo.PendoOptions createPendoOptions(@Nullable HashMap appParams) {
    Pendo.PendoOptions.Builder pendoOptions =  new Pendo.PendoOptions.Builder().setIgnoreFirstOnCreate(true);
    if (appParams != null && appParams.containsKey(VAL_ENV_NAME)) {
      String environmentName = (String) appParams.get(VAL_ENV_NAME);
      if (!isNullOrWhiteSpace(environmentName)) {
        pendoOptions.setEnvironmentDebugOnly(environmentName);
      }
    }
    return pendoOptions.build();
  }

  private static boolean isNullOrWhiteSpace(String value) {
    return value == null || value.trim().isEmpty();
  }
}
