#import "PendoFlutterPlugin.h"
#import <Pendo/Pendo.h>

typedef NS_ENUM(NSInteger, METHOD_CALLS_OPCODES){
    UNAVAILABLE              = -1,
    INIT_SDK                 = 1,
    INIT_SDK_WITHOUT_VISITOR = 2,
    CLEAR_VISITOR            = 3,
    SWITCH_VISITOR           = 4,
    SET_VISITOR_DATA         = 5,
    SET_ACCOUNT_DATA         = 6,
    TRACK                    = 7,
    GET_VISITOR_ID           = 8,
    GET_ACCOUNT_ID           = 9,
    GET_DEVICE_ID            = 10,
    SET_ACCOUNT_ID           = 11,
    END_SESSION              = 12,
    PAUSE_GUIDES             = 13,
    RESUME_GUIDES            = 14,
    DISMISS_VISIBLE_GUIDES   = 15,
    SET_DEBUG_MODE           = 16,
    INIT_SDK_SETUP           = 17,
    START_SESSION            = 18
};

static NSString *const kPNDPendoChannelName = @"pendo_sdk_channel_name";
static NSString *const kPNDPendoPluginName = @"pendo_sdk";

@implementation PendoFlutterPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:kPNDPendoChannelName binaryMessenger:[registrar messenger]];
    PendoFlutterPlugin* instance = [[PendoFlutterPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
}

+ (void)registerWithRegistry:(NSObject<FlutterPluginRegistry>*)registry {
    NSObject<FlutterPluginRegistrar> *registrar = [registry registrarForPlugin:kPNDPendoPluginName];
    [self registerWithRegistrar:registrar];
}

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSLog(@"method name: %@",call.method);
    NSUInteger methodOpcode = [self methodOpcode:call.method];
    id arguments = call.arguments;

    switch (methodOpcode) {
        case INIT_SDK: {
            [self initSDK:arguments result:result];
            break;
        } case INIT_SDK_WITHOUT_VISITOR: {
            [self initSDKWithoutVisitor:arguments result:result];
            break;
        } case CLEAR_VISITOR: {
            [self clearVisitor:arguments result:result];
            break;
        } case SWITCH_VISITOR: {
            [self switchVisitor:arguments result:result];
            break;
        } case SET_VISITOR_DATA: {
            [self setVisitorData:arguments result:result];
            break;
        } case TRACK: {
            [self track:arguments result:result];
            break;
        } case GET_VISITOR_ID: {
            [self getVisitorId:arguments result:result];
            break;
        } case GET_ACCOUNT_ID: {
            [self getAccountId:arguments result:result];
            break;
        } case GET_DEVICE_ID: {
            [self getDeviceId:arguments result:result];
            break;
        } case SET_ACCOUNT_ID: {
            [self setAccountId:arguments result:result];
            break;
        } case SET_ACCOUNT_DATA: {
            [self setAccountData:arguments result:result];
            break;
        } case END_SESSION: {
            [self endSession:arguments result:result];
            break;
        } case PAUSE_GUIDES: {
            [self pauseGuides:arguments result:result];
            break;
        } case RESUME_GUIDES: {
            [self resumeGuides:arguments result:result];
            break;
        } case DISMISS_VISIBLE_GUIDES: {
            [self dismissVisibleGuides:arguments result:result];
            break;
        } case SET_DEBUG_MODE: {
            [self setDebugMode:arguments result:result];
            break;
        } case INIT_SDK_SETUP: {
            [self setup:arguments result:result];
            break;
        } case START_SESSION: {
            [self startSession:arguments result:result];
            break;
        } default: {
            [self methodNotImplemented:arguments result:result];
            break;
        }
    }
}

#pragma mark - SDK Wrappers
- (void)initSDK:(id)args result:(FlutterResult)result {
    if (nil != args && [args isKindOfClass:[NSDictionary class]]) {
        NSDictionary *arguments = (NSDictionary *)args;
        NSString *appKey = arguments[@"ApplicationKey"];
        NSDictionary *initParams = arguments[@"ApplicationParameters"];
        [self initSDK:appKey initParams:initParams];
        result(nil);
    } else {
        FlutterError *error = [FlutterError errorWithCode:@(INIT_SDK).stringValue message:@"initSDK Call Failed" details:nil];
        result(error);
    }
}

- (void)initSDKWithoutVisitor:(id)args result:(FlutterResult)result {
    if (nil != args && [args isKindOfClass:[NSDictionary class]]) {
        NSDictionary *arguments = (NSDictionary *)args;
        NSString *appKey = arguments[@"ApplicationKey"];
        NSDictionary *sdkArgs = arguments[@"ApplicationParameters"];

        if (![sdkArgs isKindOfClass:[NSNull class]]) {
            NSString *environmentName = sdkArgs[@"environmentName"];
            BOOL disableAnalytics = [sdkArgs[@"disableAnalytics"] boolValue];
            NSDictionary *configs = sdkArgs[@"configs"];

            PendoOptions *options = [[PendoOptions alloc] init];
            options.environmentName = environmentName;
            options.disableAnalytics = disableAnalytics;
            options.configs = [configs mutableCopy];

            [self initSDKWithoutVisitor:appKey withOptions:options];
        } else {
            [self initSDKWithoutVisitor:appKey];
        }
        result(nil);
    } else {
        FlutterError *error = [FlutterError errorWithCode:@(INIT_SDK_WITHOUT_VISITOR).stringValue message:@"initSDKWithoutVisitor Call Failed" details:nil];
        result(error);
    }
}

- (void)setup:(id)args result:(FlutterResult)result {
    if (nil != args && [args isKindOfClass:[NSDictionary class]]) {
        NSDictionary *arguments = (NSDictionary *)args;
        NSString *appKey = arguments[@"ApplicationKey"];
        NSDictionary *sdkArgs = arguments[@"ApplicationParameters"];

        if (![sdkArgs isKindOfClass:[NSNull class]]) {
            NSString *environmentName = sdkArgs[@"environmentName"];
            BOOL disableAnalytics = [sdkArgs[@"disableAnalytics"] boolValue];
            NSDictionary *configs = sdkArgs[@"configs"];

            PendoOptions *options = [[PendoOptions alloc] init];
            options.environmentName = environmentName;
            options.disableAnalytics = disableAnalytics;
            options.configs = [configs mutableCopy];

            [self setup:appKey withOptions:options];
        } else {
            [self setup:appKey];
        }
        result(nil);
    } else {
        FlutterError *error = [FlutterError errorWithCode:@(INIT_SDK_SETUP).stringValue message:@"setup Call Failed" details:nil];
        result(error);
    }
}

- (void)clearVisitor:(id)args result:(FlutterResult)result {
    [self clearVisitor];
    result(nil);
}

- (void)switchVisitor:(id)args result:(FlutterResult)result {
    if (nil != args && [args isKindOfClass:[NSDictionary class]]) {
        NSDictionary *arguments = args;
        NSString *visitorId = arguments[@"VisitorIDKey"];
        if ([visitorId isKindOfClass:[NSNull class]]) {
            visitorId = nil;
        }
        NSString *accountId = arguments[@"AccountIDKey"];
        if ([accountId isKindOfClass:[NSNull class]]) {
            accountId = nil;
        }
        NSDictionary *visitorData = arguments[@"VisitorDataKey"];
        if ([visitorData isKindOfClass:[NSNull class]]) {
            visitorData = nil;
        }
        NSDictionary *accountData = arguments[@"AccountDataKey"];
        if ([accountData isKindOfClass:[NSNull class]]) {
            accountData = nil;
        }
        [self switchVisitor:visitorId accountId:accountId visitorData:visitorData accountData:accountData];
        result(nil);
    } else {
        FlutterError *error = [FlutterError errorWithCode:@(SWITCH_VISITOR).stringValue message:@"SwitchVisitor Call Failed" details:nil];
        result(error);
    }
}

- (void)startSession:(id)args result:(FlutterResult)result {
    if (nil != args && [args isKindOfClass:[NSDictionary class]]) {
        NSDictionary *arguments = args;
        NSString *visitorId = arguments[@"VisitorIDKey"];
        if ([visitorId isKindOfClass:[NSNull class]]) {
            visitorId = nil;
        }
        NSString *accountId = arguments[@"AccountIDKey"];
        if ([accountId isKindOfClass:[NSNull class]]) {
            accountId = nil;
        }
        NSDictionary *visitorData = arguments[@"VisitorDataKey"];
        if ([visitorData isKindOfClass:[NSNull class]]) {
            visitorData = nil;
        }
        NSDictionary *accountData = arguments[@"AccountDataKey"];
        if ([accountData isKindOfClass:[NSNull class]]) {
            accountData = nil;
        }
        [self startSession:visitorId accountId:accountId visitorData:visitorData accountData:accountData];
        result(nil);
    } else {
        FlutterError *error = [FlutterError errorWithCode:@(START_SESSION).stringValue message:@"startSession Call Failed" details:nil];
        result(error);
    }
}

- (void)setVisitorData:(id)args result:(FlutterResult)result {
    if (nil != args && [args isKindOfClass:[NSDictionary class]]) {
        NSDictionary *arguments = (NSDictionary *)args;
        NSDictionary *visitorData = arguments[@"VisitorDataKey"];
        [self setVisitorData:visitorData];
        result(nil);
    } else  {
        FlutterError *error = [FlutterError errorWithCode:@(SET_VISITOR_DATA).stringValue message:@"SetVisitorData Call Failed" details:nil];
        result(error);
    }
}

- (void)setAccountData:(id)args result:(FlutterResult)result {
    if (nil != args && [args isKindOfClass:[NSDictionary class]]) {
        NSDictionary *arguments = (NSDictionary *)args;
        NSDictionary *accountData = arguments[@"AccountDataKey"];
        [self setAccountData:accountData];
        result(nil);
    } else {
        FlutterError *error = [FlutterError errorWithCode:@(SET_ACCOUNT_DATA).stringValue message:@"setAccountData Call Failed" details:nil];
        result(error);
    }
}

- (void)track:(id)args result:(FlutterResult)result {
    if (nil != args && [args isKindOfClass:[NSDictionary class]]) {
        NSDictionary *arguments = (NSDictionary *)args;
        NSString *eventName = arguments[@"EventNameKey"];
        NSDictionary *properties = arguments[@"EventPropertiesKey"];
        [self track:eventName properties:properties];
        result(nil);
    } else {
        FlutterError *error = [FlutterError errorWithCode:@(TRACK).stringValue message:@"Track Call Failed" details:nil];
        result(error);
    }
}

- (void)getVisitorId:(id)args result:(FlutterResult)result {
    result([self visitorId]);
}

- (void)getAccountId:(id)args result:(FlutterResult)result {
    result([self accountId]);
}

- (void)getDeviceId:(id)args result:(FlutterResult)result {
    result([self deviceId]);
}

- (void)setAccountId:(id)args result:(FlutterResult)result {
    if (nil != args && [args isKindOfClass:[NSDictionary class]]) {
        NSDictionary *arguments = (NSDictionary *)args;
        NSString* accountId = arguments[@"AccountIDKey"];
        [self setAccountId:accountId];
        result(nil);
    } else {
        FlutterError *error = [FlutterError errorWithCode:@(SET_ACCOUNT_ID).stringValue message:@"SetAccountId Call Failed" details:nil];
        result(error);
    }
}

- (void)endSession:(id)args result:(FlutterResult)result {
    [self endSession];
    result(nil);
}

- (void)pauseGuides:(id)args result:(FlutterResult)result {
    if (nil != args && [args isKindOfClass:[NSDictionary class]]) {
        NSDictionary *arguments = (NSDictionary *)args;
        BOOL shouldDismissGuidesKey = [arguments[@"DismissGuidesKey"] boolValue];
        [self pauseGuides:shouldDismissGuidesKey];
        result(nil);
    } else {
        FlutterError *error = [FlutterError errorWithCode:@(SET_ACCOUNT_ID).stringValue message:@"PauseGuides Call Failed" details:nil];
        result(error);
    }
}

- (void)resumeGuides:(id)args result:(FlutterResult)result {
    [self resumeGuides];
    result(nil);
}

- (void)dismissVisibleGuides:(id)args result:(FlutterResult)result {
    [self dismissVisibleGuides];
    result(nil);
}

- (void)setDebugMode:(id)args result:(FlutterResult)result {
    if (nil != args && [args isKindOfClass:[NSDictionary class]]) {
        NSDictionary *arguments = (NSDictionary *)args;
        BOOL isDebugEnabled = [arguments[@"isDebugEnabled"] boolValue];
        [self setDebugMode:isDebugEnabled];
        result(nil);
    } else {
        FlutterError *error = [FlutterError errorWithCode:@(SET_DEBUG_MODE).stringValue message:@"SetDebugMode Call Failed" details:nil];
        result(error);
    }
}

- (void)methodNotImplemented:(id)args result:(FlutterResult) result {
    result(FlutterMethodNotImplemented);
}

#pragma mark - SDK methods

- (void)initSDK:(nonnull NSString *)appKey initParams:(nullable NSDictionary *)initParams {
    PendoInitParams *sdkInitParams = [[PendoInitParams alloc] init];

    if (initParams != nil) {
        if ([[initParams allKeys] containsObject:@"visitorId"]) {
            NSString *visitorId = initParams[@"visitorId"];
            [sdkInitParams setVisitorId:visitorId];
        }

        if ([[initParams allKeys] containsObject:@"accountId"]) {
            NSString *accountId = initParams[@"accountId"];
            [sdkInitParams setAccountId:accountId];
        }

        if ([[initParams allKeys] containsObject:@"visitorData"]) {
            NSDictionary *visitorData = initParams[@"visitorData"];
            [sdkInitParams setVisitorData:visitorData];
        }

        if ([[initParams allKeys] containsObject:@"accountData"]) {
            NSDictionary *accountData = initParams[@"accountData"];
            [sdkInitParams setAccountData:accountData];
        }

        if ([[initParams allKeys] containsObject:@"environmentName"]) {
            NSString *environmentName = initParams[@"environmentName"];
            PendoOptions *options = [[PendoOptions alloc] init];
            [options setEnvironmentName:environmentName];
            [sdkInitParams setOptions:options];
        }
    }

    [[PendoManager sharedManager] initSDK:appKey initParams:sdkInitParams];
}
/**
 * Must be called <b>after</b> the SDK was initialized.
 * @brief Clears the current visitor.
 */
- (void)clearVisitor {
    [[PendoManager sharedManager] clearVisitor];
}

/**
 * Must be called <b>after</b> the SDK was initialized.
 *
 * @brief Switch to a new visitor.
 *
 * @param visitorId The visitor's ID.
 * @param accountId The account's ID.
 * @param visitorData The visitor's data.
 * @param accountData The account's data.
 */
- (void)switchVisitor:(nullable NSString *)visitorId
accountId:(nullable NSString *)accountId
visitorData:(nullable NSDictionary *)visitorData
accountData:(nullable NSDictionary *)accountData {
    [[PendoManager sharedManager] switchVisitor:visitorId accountId:accountId visitorData:visitorData accountData:accountData];
}

/**
 * Must be called <b>after</b> the SDK was initialized.
 *
 * @brief start a session with a new visitor.
 * In case a session was already started before, end the previous one and start a new one.
 *
 * @param visitorId The visitor's ID.
 * @param accountId The account's ID.
 * @param visitorData The visitor's data.
 * @param accountData The account's data.
 */
- (void)startSession:(nullable NSString *)visitorId
accountId:(nullable NSString *)accountId
visitorData:(nullable NSDictionary *)visitorData
accountData:(nullable NSDictionary *)accountData {
    [[PendoManager sharedManager] startSession:visitorId accountId:accountId visitorData:visitorData accountData:accountData];
}

/**
 * Set a visitor data value.
 * This data is used by Pendo Mobile for creating audiences or reporting analytics.
 * For instance you might want to provide data on the visitor's age or if the visitor is logged into a service.
 *
 *  @param visitorData additional visitor data.
 */
- (void)setVisitorData:(nonnull NSDictionary *)visitorData {
    [[PendoManager sharedManager] setVisitorData:visitorData];
}

/**
 * Set account data value for a given data name.
 * This data is used by Pendo Mobile for creating audiences or reporting analytics.
 * For instance you might want to provide data on the account's subscription or if the account is active or not.
 *
 * @param accountData the account data.
 */
- (void)setAccountData:(nonnull NSDictionary *)accountData {
    [[PendoManager sharedManager] setAccountData:accountData];
}

/**
 * When your application needs to send additional events about actions your users perform.
 *
 * @param event The event name describing the user’s action.
 * @param properties dictionary of event properties (optional).
 * @brief Queue a track eventsfor transmission, optionally including properties as the payload of the event.
 */
- (void)track:(nonnull NSString *)event properties:(nullable NSDictionary *)properties {
    [[PendoManager sharedManager] track:event properties:properties];
}

/**
 *  Call this method on the sharedManger with your application key.
 *  Use this API in case the account and visitor data are not yet known, and they will be known at a later point of the app's lifecycle. In order to update the account and visitor data, call the API: [PendoManager switchVisitor:accountId:visitorData:accountData];
 *
 *  @warning This API is NOT an anonymous API. this should only be used if visitor data will be known at a later point.
 *  @param appKey The app key for your account
 */
- (void)initSDKWithoutVisitor:(NSString *_Nonnull)appKey {
    [[PendoManager sharedManager] initSDKWithoutVisitor:appKey];
}

/**
 *  Call this method on the sharedManger with your application key.
 *  Use this API in case the account and visitor data are not yet known, and they will be known at a later point of the app's lifecycle. In order to update the account and visitor data, call the API: [PendoManager switchVisitor:accountId:visitorData:accountData];
 *
 *  @warning This API is NOT an anonymous API. this should only be used if visitor data will be known at a later point.
 *  @param appKey The app key for your account
 *  @param options additional options for internal use only, default nil
 */
- (void)initSDKWithoutVisitor:(NSString *)appKey withOptions:(PendoOptions *_Nullable)options {
    [[PendoManager sharedManager] initSDKWithoutVisitor:appKey withOptions:options];
}

/**
 *  Call this method on the sharedManger with your application key.
 *  Setup the Pendo SDK. Call startSession when you would like to start the session (anonymous or identified)
 *
 *  @param appKey The app key for your account
 */
- (void)setup:(NSString *_Nonnull)appKey {
    [[PendoManager sharedManager] setup:appKey];
}

/**
 *  Call this method on the sharedManger with your application key.
 *  Setup the Pendo SDK. Call startSession when you would like to start the session (anonymous or identified)
 *
 *  @param appKey The app key for your account
 *  @param options additional options for internal use only, default nil
 */
- (void)setup:(NSString *)appKey withOptions:(PendoOptions *_Nullable)options {
    [[PendoManager sharedManager] setup:appKey withOptions:options];
}

/**
 *  Provide a visitor id to the Pendo Mobile SDK.
 */
- (NSString *)visitorId {
    return [[PendoManager sharedManager] visitorId];
}

/**
 *  Provide a account id to the Pendo Mobile SDK.
 */
- (NSString *)accountId {
    return [[PendoManager sharedManager] accountId];
}

/**
 * Get Pendo SDK unique device id. Used for anonymous visitor id.
 * This id is unique per application.
 */
- (NSString *)deviceId {
    return [[PendoManager sharedManager] getDeviceId];
}

/**
 *  Call this method on the sharedManger to end current session and stop gathering analytics.
 *  @warning Must be called <b>after</b> the SDK was initialized.
 */
- (void)endSession {
    [[PendoManager sharedManager] endSession];
}

/**
 * Call in order to stop showing guides.
 * @param dismissGuides indicates if to dismiss on going guides from screen or not
 */
- (void)pauseGuides:(BOOL)dismissGuides {
    [[PendoManager sharedManager] pauseGuides:dismissGuides];
}

/**
 * Call in order to resume showing guides.
 */
- (void)resumeGuides {
    [[PendoManager sharedManager] resumeGuides];
}

/**
 * Dismiss any visible guides
 */
- (void)dismissVisibleGuides {
    [[PendoManager sharedManager] dismissVisibleGuides];
}

/**
 * Set debug mode to true to display log messages
 */
- (void)setDebugMode:(BOOL)isDebugEnabled {
    [[PendoManager sharedManager] setDebugMode:isDebugEnabled];
}

/**
 * Set the current accountId to a new one
 */
- (void)setAccountId:(NSString*)accountId {
    [PendoManager sharedManager].accountId = accountId;
}

- (NSInteger)methodOpcode:(NSString *)methodName {
    if (methodName.length == 0) {
        return UNAVAILABLE;
    }
    NSDictionary <NSString*, NSNumber *> *methodNamesOpcode = [self methodNamesOpcodes];
    return methodNamesOpcode[methodName].intValue ?: UNAVAILABLE;
}


- (NSDictionary <NSString*, NSNumber *>*)methodNamesOpcodes {
    static NSDictionary *methodNames = nil;
    static dispatch_once_t once;
    dispatch_once(&once,^{
        methodNames =  @{
            @"initSDK":                @(INIT_SDK),
            @"initSDKWithoutVisitor":  @(INIT_SDK_WITHOUT_VISITOR),
            @"clearVisitor":           @(CLEAR_VISITOR),
            @"switchVisitor":          @(SWITCH_VISITOR),
            @"setVisitorData":         @(SET_VISITOR_DATA),
            @"setAccountData":         @(SET_ACCOUNT_DATA),
            @"track":                  @(TRACK),
            @"getVisitorId":           @(GET_VISITOR_ID),
            @"getAccountId":           @(GET_ACCOUNT_ID),
            @"getDeviceId":            @(GET_DEVICE_ID),
            @"setAccountId":           @(SET_ACCOUNT_ID),
            @"endSession":             @(END_SESSION),
            @"pauseGuides":            @(PAUSE_GUIDES),
            @"resumeGuides":           @(RESUME_GUIDES),
            @"dismissVisibleGuides":   @(DISMISS_VISIBLE_GUIDES),
            @"setDebugMode":           @(SET_DEBUG_MODE),
            @"setup":                   @(INIT_SDK_SETUP),
            @"startSession":            @(START_SESSION)
        };
    });

    return methodNames;
}

@end
