import 'dart:async';
import 'package:flutter/services.dart';

class PendoFlutterPlugin {
  static const MethodChannel _channel =
      const MethodChannel('pendo_sdk_channel_name');

  @Deprecated(
      'Please follow the SDK guideline by using the new APIs as documented in Pendo API documentation https://support.pendo.io/hc/en-us/articles/4405623533979')

  /// Initializes the Pendo SDK and starts a session.
  ///
  /// Use the [appKey] to init the SDK.
  ///
  /// Use [initParams] for extra initialize parameters (e.g visitor id, account id)
  ///
  /// Example usage:
  /// `PendoFlutterPlugin.initSDK('YOUR_PENDO_KEY', {'visitorId: 'some visitor', 'accountId': 'some account'});`
  static Future<void> initSDK(
      String appKey, Map<String, dynamic>? initParams) async {
    try {
      var arguments = {
        "ApplicationKey": appKey,
        "ApplicationParameters": initParams
      };
      await _channel.invokeMethod('initSDK', arguments);
    } catch (e) {
      throw ('Failed calling initSDK() with appKey: $appKey, initParams: $initParams, error is: $e');
    }
  }

  @Deprecated(
      'Please follow the SDK guideline by using the new APIs as documented in Pendo API documentation https://support.pendo.io/hc/en-us/articles/4405623533979')

  /// Initializes the Pendo SDK
  ///
  /// Use the [appKey] to init the SDK.
  /// Setting up the SDK <b>does not</b> start a session.
  ///
  /// [appParams] is for internal use and is optional.
  ///
  /// Example usage:
  /// `PendoFlutterPlugin.initSDKWithoutVisitor('YOUR_PENDO_KEY');`
  ///
  /// Call the `switchVisitor` API when you would like to start tracking analytics and/or presenting guides (for anonymous or identified visitors).
  static Future<void> initSDKWithoutVisitor(String appKey,
      [Map<String, dynamic>? appParams]) async {
    try {
      var arguments = {
        "ApplicationKey": appKey,
        "ApplicationParameters": appParams
      };
      await _channel.invokeMethod('initSDKWithoutVisitor', arguments);
    } catch (e) {
      throw ('Failed calling initSDKWithoutVisitor() with appKey: $appKey, appParams: $appParams, error is: $e');
    }
  }

  /// Setups the Pendo SDK.
  ///
  /// Use the [appKey] to setup the SDK.
  /// Setting up the SDK <b>does not</b> start a session.
  ///
  /// [appParams] is for internal use and is optional.
  ///
  /// Example usage:
  /// `PendoFlutterPlugin.setup('YOUR_PENDO_KEY');`
  ///
  /// Call the `startSession` API when you would like to start tracking analytics and/or presenting guides (for anonymous or identified visitors).
  static Future<void> setup(String appKey,
      [Map<String, dynamic>? appParams]) async {
    try {
      var arguments = {
        "ApplicationKey": appKey,
        "ApplicationParameters": appParams
      };
      await _channel.invokeMethod('setup', arguments);
    } catch (e) {
      throw ('Failed calling setup() with appKey: $appKey, appParams: $appParams, error is: $e');
    }
  }

  /// Must be called <b>after</b> the SDK was setup.
  ///
  /// Clears the current visitor.
  ///
  /// Stops the current session and starts a new session with an anonymous visitor.
  static Future<void> clearVisitor() async {
    try {
      await _channel.invokeMethod('clearVisitor');
    } catch (e) {
      throw ('Failed calling clearVisitor(), error is: $e');
    }
  }

  @Deprecated(
      'Please follow the SDK guideline by using the new APIs as documented in Pendo API documentation https://support.pendo.io/hc/en-us/articles/4405623533979')

  /// Must be called <b>after</b> the SDK was setup.
  ///
  /// Starts a session with a new visitor.
  /// In case a session was already started before, end the previous one and start a new one.
  static Future<void> switchVisitor(
      String? visitorId,
      String? accountId,
      Map<String, dynamic>? visitorData,
      Map<String, dynamic>? accountData) async {
    try {
      var arguments = {
        "VisitorIDKey": visitorId,
        "AccountIDKey": accountId,
        "VisitorDataKey": visitorData,
        "AccountDataKey": accountData
      };
      await _channel.invokeMethod('switchVisitor', arguments);
    } catch (e) {
      throw ('Failed calling switchVisitor() with visitorId: $visitorId, accountId: $accountId, visitorData: $visitorData, accountData: $accountData, error is: $e');
    }
  }

  /// Must be called <b>after</b> the SDK was setup.
  ///
  /// Starts a session with a new visitor.
  /// In case a session was already started before, end the previous one and start a new one.
  static Future<void> startSession(
      String? visitorId,
      String? accountId,
      Map<String, dynamic>? visitorData,
      Map<String, dynamic>? accountData) async {
    try {
      var arguments = {
        "VisitorIDKey": visitorId,
        "AccountIDKey": accountId,
        "VisitorDataKey": visitorData,
        "AccountDataKey": accountData
      };
      await _channel.invokeMethod('startSession', arguments);
    } catch (e) {
      throw ('Failed calling startSession() with visitorId: $visitorId, accountId: $accountId, visitorData: $visitorData, accountData: $accountData, error is: $e');
    }
  }

  /// Sets a visitor data value.
  ///
  /// This data is used by Pendo Mobile for creating audiences or reporting analytics.
  ///
  /// For instance you might want to provide data on the visitor's age or if the visitor is logged into a service.
  static Future<void> setVisitorData(Map<String, dynamic>? visitorData) async {
    try {
      var arguments = {"VisitorDataKey": visitorData};
      await _channel.invokeMethod('setVisitorData', arguments);
    } catch (e) {
      throw ('Failed calling setVisitorData() with visitorData: $visitorData, error is: $e');
    }
  }

  /// Sets account data value for a given data name.
  ///
  /// This data is used by Pendo Mobile for creating audiences or reporting analytics.
  ///
  /// For instance you might want to provide data on the account's subscription or if the account is active or not.
  static Future<void> setAccountData(Map<String, dynamic>? accountData) async {
    try {
      var arguments = {"AccountDataKey": accountData};
      await _channel.invokeMethod('setAccountData', arguments);
    } catch (e) {
      throw ('Failed calling setAccountData() with accountData: $accountData, error is: $e');
    }
  }

  /// Tracks an event that has happened in your application.
  static Future<void> track(
      String event, Map<String, dynamic>? properties) async {
    try {
      var arguments = {"EventNameKey": event, "EventPropertiesKey": properties};
      await _channel.invokeMethod('track', arguments);
    } catch (e) {
      throw ('Failed calling track() with event: $event, properties: $properties, error is: $e');
    }
  }

  /// Must be called <b>after</b> the SDK was setup.
  ///
  /// Ends the current session. As a result analytics gathering and presentation of guides will stop.
  static Future<void> endSession() async {
    try {
      await _channel.invokeMethod('endSession');
    } catch (e) {
      throw ('Failed calling endSession(), error is: $e');
    }
  }

  /// Sets the account id.
  static Future<void> setAccountId(String? accountId) async {
    try {
      var arguments = {"AccountIDKey": accountId};
      await _channel.invokeMethod('setAccountId', arguments);
    } catch (e) {
      throw ('Failed calling setAccountId() with accountId: $accountId, error is: $e');
    }
  }

  /// Stops showing guides.
  ///
  /// Setting [dismissGuides] to true will close any guide currently presented on the screen.
  static Future<void> pauseGuides(bool dismissGuides) async {
    try {
      Map<String, bool> arguments = {"DismissGuidesKey": dismissGuides};
      await _channel.invokeMethod('pauseGuides', arguments);
    } catch (e) {
      throw ('Failed calling pauseGuides() with dismissGuides: $dismissGuides, error is: $e');
    }
  }

  /// Dismisses any visible guides.
  static Future<void> dismissVisibleGuides() async {
    try {
      await _channel.invokeMethod('dismissVisibleGuides');
    } catch (e) {
      throw ('Failed calling dismissVisibleGuides(), error is: $e');
    }
  }

  /// Resumes presenting guides.
  static Future<void> resumeGuides() async {
    try {
      await _channel.invokeMethod('resumeGuides');
    } catch (e) {
      throw ('Failed calling resumeGuides(), error is: $e');
    }
  }

  /// Returns the current visitor id if available.
  static Future<String?> getVisitorId() async {
    try {
      return await _channel.invokeMethod('getVisitorId');
    } catch (e) {
      throw ('Failed calling getVisitorId(), error is: $e');
    }
  }

  /// Returns the current account id if available.
  static Future<String?> getAccountId() async {
    try {
      return await _channel.invokeMethod<String>('getAccountId');
    } catch (e) {
      throw ('Failed calling getAccountId(), error is: $e');
    }
  }

  /// Returns the Pendo SDK unique device id.
  ///
  /// Used for anonymous visitor id.
  /// This id is unique per application.
  static Future<String?> getDeviceId() async {
    try {
      return await _channel.invokeMethod<String>('getDeviceId');
    } catch (e) {
      throw ('Failed calling getDeviceId(), error is: $e');
    }
  }

  /// Presents debug logs when [isDebugEnabled] is set to true.
  static Future<void> setDebugMode(bool isDebugEnabled) async {
    try {
      Map<String, bool> arguments = {"isDebugEnabled": isDebugEnabled};
      await _channel.invokeMethod('setDebugMode', arguments);
    } catch (e) {
      throw ('Failed calling setDebugMode() with isDebugEnabled: $isDebugEnabled, error is: $e');
    }
  }
}
